@@ -13,7 +13,6 @@ We can also move the implementation to C++ if desired.
 """
 
 import inspect
-from collections import namedtuple
 from collections.abc import KeysView
 from io import BytesIO
 from subprocess import PIPE, run
@@ -21,7 +20,7 @@ from tempfile import NamedTemporaryFile
 
 from . import Array, Dictionary, Name, Object, Page, Pdf, Stream
 from ._qpdf import PdfError, StreamParser, Token, _ObjectMapping
-from .models import EncryptionInfo, PdfMetadata, Permissions
+from .models import EncryptionInfo, Outline, PdfMetadata, Permissions
 
 # pylint: disable=no-member,unsupported-membership-test,unsubscriptable-object
 
@@ -100,7 +99,10 @@ def _mudraw(buffer, fmt):
         tmp_in.flush()
 
         proc = run(
-            ['mudraw', '-F', fmt, '-o', '-', tmp_in.name], stdout=PIPE, stderr=PIPE
+            ['mudraw', '-F', fmt, '-o', '-', tmp_in.name],
+            stdout=PIPE,
+            stderr=PIPE,
+            check=True,
         )
         if proc.stderr:
             raise RuntimeError(proc.stderr.decode())
@@ -124,10 +126,8 @@ class Extend_Object:
         return None
 
     def _ipython_key_completions_(self):
-        if isinstance(self, Dictionary):
+        if isinstance(self, (Dictionary, Stream)):
             return self.keys()
-        elif isinstance(self, Stream):
-            return self.stream_dict.keys()
         return None
 
     def emplace(self, other):
@@ -155,13 +155,17 @@ class Extend_Object:
             >>> pdf.pages[0].objgen
             (16, 0)  # Same object
         """
+        if not self.same_owner_as(other):
+            raise TypeError("Objects must have the same owner for emplace()")
         del_keys = set(self.keys()) - set(other.keys())
         for k in other.keys():
             self[k] = other[k]  # pylint: disable=unsupported-assignment-operation
         for k in del_keys:
             del self[k]  # pylint: disable=unsupported-delete-operation
 
-    def write(self, data, *, filter=None, decode_parms=None, type_check=True):
+    def write(
+        self, data, *, filter=None, decode_parms=None, type_check=True
+    ):  # pylint: disable=redefined-builtin
         """
         Replace stream object's data with new (possibly compressed) `data`.
 
@@ -255,10 +259,17 @@ class Extend_Pdf:
         self, set_pikepdf_as_editor=True, update_docinfo=True, strict=False
     ):
         """
-        Open the PDF's XMP metadata for editing
+        Open the PDF's XMP metadata for editing.
 
-        Recommend for use in a ``with`` block. Changes are committed to the
-        PDF when the block exits. (The ``Pdf`` must still be opened.)
+        There is no ``.close()`` function on the metadata object, since this is
+        intended to be used inside a ``with`` block only.
+
+        For historical reasons, certain parts of PDF metadata are stored in
+        two different locations and formats. This feature coordinates edits so
+        that both types of metadata are updated consistently and "atomically"
+        (assuming single threaded access). It operates on the ``Pdf`` in memory,
+        not any file on disk. To persist metadata changes, you must still use
+        ``Pdf.save()``.
 
         Example:
             >>> with pdf.open_metadata() as meta:
@@ -270,8 +281,12 @@ class Extend_Pdf:
                 version of pikepdf is the most recent software to modify the metadata.
                 Recommended, except for testing.
 
-            update_docinfo (bool): Update the deprecated PDF DocumentInfo block
-                to be consistent with XMP.
+            update_docinfo (bool): Update the standard fields of DocumentInfo
+                (the old PDF metadata dictionary) to match the corresponding
+                XMP fields. The mapping is described in
+                :attr:`PdfMetadata.DOCINFO_MAPPING`. Nonstandard DocumentInfo
+                fields and XMP metadata fields with no DocumentInfo equivalent
+                are ignored.
 
             strict (bool): If ``False`` (the default), we aggressively attempt
                 to recover from any parse errors in XMP, and if that fails we
@@ -292,6 +307,40 @@ class Extend_Pdf:
             overwrite_invalid_xml=not strict,
         )
 
+    def open_outline(self, max_depth=15, strict=False):
+        """
+        Open the PDF outline ("bookmarks") for editing.
+
+        Recommend for use in a ``with`` block. Changes are committed to the
+        PDF when the block exits. (The ``Pdf`` must still be opened.)
+
+        Example:
+            >>> with pdf.open_outline() as outline:
+                    outline.root.insert(0, OutlineItem('Intro', 0))
+
+        Args:
+            max_depth (int): Maximum recursion depth of the outline to be
+                imported and re-written to the document. ``0`` means only
+                considering the root level, ``1`` the first-level
+                sub-outline of each root element, and so on. Items beyond
+                this depth will be silently ignored. Default is ``15``.
+            strict (bool): With the default behavior (set to ``False``),
+                structural errors (e.g. reference loops) in the PDF document
+                will only cancel processing further nodes on that particular
+                level, recovering the valid parts of the document outline
+                without raising an exception. When set to ``True``, any such
+                error will raise an ``OutlineStructureError``, leaving the
+                invalid parts in place.
+                Similarly, outline objects that have been accidentally
+                duplicated in the ``Outline`` container will be silently
+                fixed (i.e. reproduced as new objects) or raise an
+                ``OutlineStructureError``.
+
+        Returns:
+            pikepdf.models.Outline
+        """
+        return Outline(self, max_depth=max_depth, strict=strict)
+
     def make_stream(self, data):
         """
         Create a new pikepdf.Stream object that is attached to this PDF.
@@ -595,7 +644,7 @@ class Extend_Page:
     def __repr__(self):
         return repr(self.obj).replace('Dictionary', 'Page')
 
-    def _repr_mimebundle_(self, include, exclude, **kwargs):
+    def _repr_mimebundle_(self, include, exclude, **_kwargs):
         data = {}
         bundle = {'application/pdf', 'image/png'}
         if include:
