@@ -11,11 +11,20 @@ from io import BytesIO
 from itertools import zip_longest
 from pathlib import Path
 from shutil import copyfileobj
-from tempfile import NamedTemporaryFile
 from zlib import decompress
 from zlib import error as ZlibError
 
-from .. import Array, Dictionary, Name, Object, PdfError, Stream
+from pikepdf import (
+    Array,
+    Dictionary,
+    Name,
+    Object,
+    PdfError,
+    Stream,
+    StreamDecodeLevel,
+    String,
+    jbig2,
+)
 
 
 class DependencyError(Exception):
@@ -31,26 +40,24 @@ class NotExtractableError(Exception):
 
 
 def array_str(value):
-    if isinstance(value, (list, Array)):
-        return [str(item) for item in value]
-    if isinstance(value, Name):
-        return [str(value)]
-    raise NotImplementedError(value)
+    """Simplify pikepdf objects to array of str. Keep Streams intact."""
 
+    def _array_str(item):
+        if isinstance(item, (list, Array)):
+            return [_array_str(subitem) for subitem in item]
+        elif isinstance(item, (Stream, bytes, int)):
+            return item
+        elif isinstance(item, (Name, str)):
+            return str(item)
+        elif isinstance(item, (String)):
+            return bytes(item)
+        else:
+            raise NotImplementedError(value)
 
-def array_str_colorspace(value):
-    if isinstance(value, (list, Array)):
-        items = [item for item in value]
-        if len(items) == 4 and items[0] == '/Indexed':
-            result = [str(items[n]) for n in range(3)]
-            result.append(bytes(items[3]))
-            return result
-        if len(items) == 2 and items[0] == '/ICCBased':
-            result = [str(items[0]), items[1]]
-            return result
-        return array_str(items)
-
-    return array_str(value)
+    result = _array_str(value)
+    if not isinstance(result, list):
+        result = [result]
+    return result
 
 
 def dict_or_array_dict(value):
@@ -75,7 +82,8 @@ def metadata_from_obj(obj, name, type_, default):
 
 class PdfImageBase(ABC):
 
-    SIMPLE_COLORSPACES = ('/DeviceRGB', '/DeviceGray', '/CalRGB', '/CalGray')
+    SIMPLE_COLORSPACES = {'/DeviceRGB', '/DeviceGray', '/CalRGB', '/CalGray'}
+    MAIN_COLORSPACES = SIMPLE_COLORSPACES | {'/DeviceCMYK', '/CalCMYK', '/ICCBased'}
 
     @abstractmethod
     def _metadata(self, name, type_, default):
@@ -104,7 +112,7 @@ class PdfImageBase(ABC):
     @property
     def _colorspaces(self):
         """Colorspace (low-level)"""
-        return self._metadata('ColorSpace', array_str_colorspace, [])
+        return self._metadata('ColorSpace', array_str, [])
 
     @property
     def filters(self):
@@ -122,15 +130,14 @@ class PdfImageBase(ABC):
         if self.image_mask:
             return None  # Undefined for image masks
         if self._colorspaces:
-            if self._colorspaces[0] in self.SIMPLE_COLORSPACES:
+            if self._colorspaces[0] in self.MAIN_COLORSPACES:
                 return self._colorspaces[0]
-            if self._colorspaces[0] in ('/DeviceCMYK', '/ICCBased'):
-                return self._colorspaces[0]
-            if (
-                self._colorspaces[0] == '/Indexed'
-                and self._colorspaces[1] in self.SIMPLE_COLORSPACES
-            ):
-                return self._colorspaces[1]
+            if self._colorspaces[0] == '/Indexed':
+                subspace = self._colorspaces[1]
+                if isinstance(subspace, str) and subspace in self.MAIN_COLORSPACES:
+                    return subspace
+                if isinstance(subspace, list) and subspace[0] == '/ICCBased':
+                    return subspace[0]
         raise NotImplementedError(
             "not sure how to get colorspace: " + repr(self._colorspaces)
         )
@@ -222,20 +229,33 @@ class PdfImageBase(ABC):
 
         if not self.indexed:
             return None
-        _idx, base, hival, lookup = None, None, None, None
         try:
-            _idx, base, hival, lookup = self._colorspaces
+            _idx, base, _hival, lookup = self._colorspaces
         except ValueError as e:
             raise ValueError('Not sure how to interpret this palette') from e
-        base = str(base)
-        hival = int(hival)
+        iccobj = None
+        if self.icc:
+            iccobj = base[1]
+            base = str(base[0])
+        else:
+            base = str(base)
+        _hival = int(_hival)
         lookup = bytes(lookup)
-        if not base in self.SIMPLE_COLORSPACES:
+        if not base in self.MAIN_COLORSPACES:
             raise NotImplementedError("not sure how to interpret this palette")
         if base == '/DeviceRGB':
             base = 'RGB'
         elif base == '/DeviceGray':
             base = 'L'
+        elif base == '/DeviceCMYK':
+            base = 'CMYK'
+        elif base == '/ICCBased':
+            if iccobj['/N'] == 3:
+                base = 'RGB'
+            elif iccobj['/N'] == 4:
+                base = 'CMYK'
+            elif iccobj['/N'] == 1:
+                base = 'L'
         return base, lookup
 
     @abstractmethod
@@ -334,6 +354,15 @@ class PdfImage(PdfImageBase):
         """``False`` for image XObject"""
         return False
 
+    @property
+    def _iccstream(self):
+        if self.colorspace == '/ICCBased':
+            if not self.indexed:
+                return self._colorspaces[1]
+            assert isinstance(self._colorspaces[1], list)
+            return self._colorspaces[1][1]
+        raise NotImplementedError("Don't know how to find ICC stream for image")
+
     @property
     def icc(self):
         """If an ICC profile is attached, return a Pillow object that describe it.
@@ -345,10 +374,10 @@ class PdfImage(PdfImageBase):
         """
         from PIL import ImageCms
 
-        if self.colorspace != '/ICCBased':
+        if self.colorspace not in ('/ICCBased', '/Indexed'):
             return None
         if not self._icc:
-            iccstream = self._colorspaces[1]
+            iccstream = self._iccstream
             iccbuffer = iccstream.get_stream_buffer()
             iccbytesio = BytesIO(iccbuffer)
             self._icc = ImageCms.ImageCmsProfile(iccbytesio)
@@ -420,14 +449,20 @@ class PdfImage(PdfImageBase):
                     im.putpalette(palette, rawmode=base_mode)
                 else:
                     raise NotImplementedError('palette with ' + base_mode)
-        elif self.mode == '1' and self.bits_per_component == 1:
-            data = self.read_bytes()
-            im = Image.frombytes('1', self.size, data)
-
-        elif self.mode == 'P' and self.bits_per_component == 1:
-            data = self.read_bytes()
-            im = Image.frombytes('1', self.size, data)
+        elif self.bits_per_component == 1:
+            if self.filters and self.filters[0] == '/JBIG2Decode':
+                if not jbig2.jbig2dec_available():
+                    raise DependencyError("jbig2dec - not installed")
+                jbig2_globals_obj = self.filter_decodeparms[0][1].get('/JBIG2Globals')
+                im = jbig2.extract_jbig2(self.obj, jbig2_globals_obj)
+            else:
+                data = self.read_bytes()
+                im = Image.frombytes('1', self.size, data)
+        else:
+            raise UnsupportedImageTypeError(repr(self))
 
+        if self.mode == 'P' and self.bits_per_component == 1:
+            # Fix paletted 1-bit images
             base_mode, palette = self.palette
             if base_mode == 'RGB' and palette != b'\x00\x00\x00\xff\xff\xff':
                 im = im.convert('P')
@@ -448,18 +483,13 @@ class PdfImage(PdfImageBase):
         return im
 
     def _extract_to_stream(self, *, stream):
-        """Attempt to extract the image directly to a usable image file
+        """Attempt to extract the image to a stream
 
         If possible, the compressed data is extracted and inserted into
         a compressed image file format without transcoding the compressed
         content. If this is not possible, the data will be decompressed
         and extracted to an appropriate format.
 
-        Because it is not known until attempted what image format will be
-        extracted, users should not assume what format they are getting back.
-        When saving the image to a file, use a temporary filename, and then
-        rename the file to its final name based on the returned file extension.
-
         Args:
             stream: Writable stream to write data to
 
@@ -532,13 +562,13 @@ class PdfImage(PdfImageBase):
             copyfileobj(bio, target)
         return str(filepath)
 
-    def read_bytes(self):
+    def read_bytes(self, decode_level=StreamDecodeLevel.specialized):
         """Decompress this image and return it as unencoded bytes"""
-        return self.obj.read_bytes()
+        return self.obj.read_bytes(decode_level=decode_level)
 
-    def get_stream_buffer(self):
+    def get_stream_buffer(self, decode_level=StreamDecodeLevel.specialized):
         """Access this image with the buffer protocol"""
-        return self.obj.get_stream_buffer()
+        return self.obj.get_stream_buffer(decode_level=decode_level)
 
     def as_pil_image(self):
         """Extract the image as a Pillow Image, using decompression as necessary
@@ -762,7 +792,7 @@ class PdfInlineImage(PdfImageBase):
 
     @property
     def icc(self):
-        raise ValueError("Inline images may not have ICC profiles")
+        raise NotImplementedError("Inline images with ICC profiles are not supported")
 
     def __repr__(self):
         mode = '?'
