@@ -11,3 +11,9 @@
 #include "pikepdf.h"
 
 py::object fspath(py::object filename);
+
+template <typename T, typename S>
+inline bool str_startswith(T haystack, S needle)
+{
+    return std::string(haystack).rfind(needle, 0) == 0;
+}
