@@ -4,7 +4,6 @@ from pathlib import Path
 
 import pytest
 
-
 if sys.version_info < (3, 4):
     print("Requires Python 3.4+")
     sys.exit(1)
@@ -14,7 +13,7 @@ TESTS_ROOT = os.path.abspath(os.path.dirname(__file__))
 PROJECT_ROOT = os.path.dirname(TESTS_ROOT)
 
 
-@pytest.fixture
+@pytest.fixture(scope="session")
 def resources():
     return Path(TESTS_ROOT) / 'resources'
 
