@@ -3,9 +3,10 @@ from io import BytesIO
 from pathlib import Path
 
 import pytest
-from PIL import Image
+from PIL import Image, ImageCms
 from PIL import features as PIL_features
 
+import pikepdf
 from pikepdf import (
     Dictionary,
     Name,
@@ -18,7 +19,11 @@ from pikepdf import (
     parse_content_stream,
 )
 from pikepdf._cpphelpers import fspath
-from pikepdf.models.image import NotExtractableError, UnsupportedImageTypeError
+from pikepdf.models.image import (
+    DependencyError,
+    NotExtractableError,
+    UnsupportedImageTypeError,
+)
 
 # pylint: disable=redefined-outer-name
 
@@ -39,6 +44,11 @@ def sandwich(resources):
     return first_image_in(resources / 'sandwich.pdf')
 
 
+@pytest.fixture
+def jbig2(resources):
+    return first_image_in(resources / 'jbig2.pdf')
+
+
 def test_image_from_nonimage(resources):
     pdf = Pdf.open(resources / 'congress.pdf')
     resources = pdf.pages[0].Contents
@@ -321,12 +331,30 @@ def test_icc_use(resources):
 
 
 def test_icc_extract(resources):
-    xobj, _pdf = first_image_in(resources / 'tree-icc.pdf')
+    xobj, _pdf = first_image_in(resources / 'aquamarine-cie.pdf')
 
     pim = PdfImage(xobj)
     assert pim.as_pil_image().info['icc_profile'] == pim.icc.tobytes()
 
 
+def test_icc_palette(resources):
+    xobj, _pdf = first_image_in(resources / 'pink-palette-icc.pdf')
+    pim = PdfImage(xobj)
+    assert pim.icc.profile.xcolor_space == 'RGB '  # with trailing space
+    b = BytesIO()
+    pim.extract_to(stream=b)
+    b.seek(0)
+
+    im = Image.open(b)
+    assert im.size == (xobj.Width, xobj.Height)
+    assert im.mode == 'P'
+    pil_icc = im.info.get('icc_profile')
+    pil_icc_stream = BytesIO(pil_icc)
+    pil_prf = ImageCms.ImageCmsProfile(pil_icc_stream)
+
+    assert pil_prf.tobytes() == pim.icc.tobytes()
+
+
 def test_stacked_compression(resources):
     xobj, _pdf = first_image_in(resources / 'pike-flate-jp2.pdf')
 
@@ -363,3 +391,47 @@ def test_imagemagick_uses_rle_compression(resources):
     pim = PdfImage(xobj)
     im = pim.as_pil_image()
     assert im.getpixel((5, 5)) == (255, 128, 0)
+
+
+def test_jbig2_not_available(jbig2, monkeypatch):
+    xobj, _pdf = jbig2
+    pim = PdfImage(xobj)
+
+    monkeypatch.setattr(pikepdf.jbig2, 'jbig2dec_available', lambda: False)
+    with pytest.raises(DependencyError):
+        pim.as_pil_image()
+
+
+needs_jbig2dec = pytest.mark.skipif(
+    not pikepdf.jbig2.jbig2dec_available(), reason="jbig2dec not installed"
+)
+
+
+@needs_jbig2dec
+def test_jbig2(jbig2):
+    xobj, _pdf = jbig2
+    pim = PdfImage(xobj)
+    im = pim.as_pil_image()
+    assert im.size == (1000, 1520)
+    assert im.getpixel((0, 0)) == 0  # Ensure loaded
+
+
+@needs_jbig2dec
+def test_jbig2_global(resources):
+    xobj, _pdf = first_image_in(resources / 'jbig2global.pdf')
+    pim = PdfImage(xobj)
+    im = pim.as_pil_image()
+    assert im.size == (4000, 2864)
+    assert im.getpixel((0, 0)) == 255  # Ensure loaded
+
+
+@needs_jbig2dec
+def test_jbig2_global_palette(resources):
+    xobj, _pdf = first_image_in(resources / 'jbig2global.pdf')
+    xobj.ColorSpace = pikepdf.Array(
+        [Name.Indexed, Name.DeviceRGB, 1, b'\x00\x00\x00\xff\xff\xff']
+    )
+    pim = PdfImage(xobj)
+    im = pim.as_pil_image()
+    assert im.size == (4000, 2864)
+    assert im.getpixel((0, 0)) == 255  # Ensure loaded
